@@ -1,6 +1,6 @@
 package Catmandu::Importer::getJSON;
 
-our $VERSION = '0.50';
+our $VERSION = '0.51';
 our $CACHE;
 
 use Catmandu::Sane;
@@ -12,46 +12,46 @@ use URI::Template;
 
 with 'Catmandu::Importer';
 
-has url     => ( 
-    is  => 'rw', 
+has url => (
+    is      => 'rw',
     trigger => sub {
-        $_[0]->{url} = _url_template_or_url($_[1])
+        $_[0]->{url} = _url_template_or_url( $_[1] );
     }
 );
 
-has from    => ( is => 'ro');
+has from    => ( is => 'ro' );
 has timeout => ( is => 'ro', default => sub { 10 } );
 has agent   => ( is => 'ro' );
 has proxy   => ( is => 'ro' );
 has dry     => ( is => 'ro' );
-has headers => ( 
-    is => 'ro', 
-    default => sub { [ 'Accept' => 'application/json' ] } 
+has headers => (
+    is      => 'ro',
+    default => sub { [ 'Accept' => 'application/json' ] }
 );
-has wait    => ( is => 'ro' );
-has cache   => ( is => 'ro', trigger => 1 );
-has client  => (
-    is => 'ro',
-    lazy => 1,
-    builder => sub { 
-        Furl->new( 
+has wait => ( is => 'ro' );
+has cache => ( is => 'ro', trigger => 1 );
+has client => (
+    is      => 'ro',
+    lazy    => 1,
+    builder => sub {
+        Furl->new(
             map { $_ => $_[0]->{$_} } grep { defined $_[0]->{$_} }
-            qw(timeout agent proxy),
-        ) 
+              qw(timeout agent proxy),
+        );
     }
 );
 has json => ( is => 'ro', default => sub { JSON->new->utf8(1) } );
 has time => ( is => 'rw' );
-has warn => ( is => 'ro', default => sub { 1 } );
+has warn => ( is => 'ro', default => sub { 0 } );
 
 sub _url_template_or_url {
     my ($url) = @_;
 
-    if (!blessed $url) {
+    if ( !blessed $url) {
         $url = URI::Template->new($url);
     }
 
-    if ($url->isa('URI::Template')) {
+    if ( $url->isa('URI::Template') ) {
         unless ( my @variables = $url->variables ) {
             $url = URI->new("$url");
         }
@@ -59,48 +59,61 @@ sub _url_template_or_url {
     return $url;
 }
 
-
 {
+
     package Importer::getJSON::MemoryCache;
     use JSON;
     our $JSON = JSON->new->utf8;
     sub new { bless {}, $_[0] }
-    sub get { eval { $JSON->decode($_[0]->{$_[1]}) } }
-    sub set { $_[0]->{$_[1]} = ref $_[2] ? $JSON->encode($_[2]) : '' }
+
+    sub get {
+        eval { $JSON->decode( $_[0]->{ $_[1] } ) };
+    }
+    sub set { $_[0]->{ $_[1] } = ref $_[2] ? $JSON->encode( $_[2] ) : '' }
 }
 $CACHE = Importer::getJSON::MemoryCache->new;
 
 {
+
     package Importer::getJSON::FileCache;
     use JSON;
     use Catmandu::Util qw(read_json);
     use Digest::MD5 qw(md5_hex);
     our $JSON = JSON->new->utf8;
+
     sub new {
-        my ($class, $dir) = @_;
+        my ( $class, $dir ) = @_;
         $dir =~ s{/$}{};
-        bless { dir => $dir }, $class 
+        bless { dir => $dir }, $class;
     }
+
     sub file {
-        my ($self, $url) = @_;
-        $self->{dir}.'/'.md5_hex($url).'.json';
+        my ( $self, $url ) = @_;
+        $self->{dir} . '/' . md5_hex($url) . '.json';
     }
-    sub get { eval { read_json($_[0]->file($_[1])) } }
-    sub set { 
-        my ($self, $url, $data) = @_;
+
+    sub get {
+        eval { read_json( $_[0]->file( $_[1] ) ) };
+    }
+
+    sub set {
+        my ( $self, $url, $data ) = @_;
         open my $fh, ">", $self->file($url);
-        print $fh (ref $data ? $JSON->encode($data) : '');
+        print $fh ( ref $data ? $JSON->encode($data) : '' );
     }
 }
 
 sub _trigger_cache {
-    my ($self, $cache) = @_;
- 
-    if (blessed $cache and $cache->can('get') and $cache->can('set')) {
-        # use cache object 
-    } elsif ($cache and -d $cache) {
+    my ( $self, $cache ) = @_;
+
+    if ( blessed $cache and $cache->can('get') and $cache->can('set') ) {
+
+        # use cache object
+    }
+    elsif ( $cache and -d $cache ) {
         $cache = Importer::getJSON::FileCache->new($cache);
-    } elsif ($cache) {
+    }
+    elsif ($cache) {
         $cache = $CACHE;
     }
 
@@ -109,15 +122,15 @@ sub _trigger_cache {
 
 sub generator {
     my ($self) = @_;
-    
-    if ($self->from) {
+
+    if ( $self->from ) {
         return sub {
             state $data = do {
-                my $r = $self->request($self->from);
-                (ref $r // '') eq 'ARRAY' ? $r : [$r];
+                my $r = $self->request( $self->from );
+                ( ref $r // '' ) eq 'ARRAY' ? $r : [$r];
             };
             return shift @$data;
-        }
+          }
     }
 
     sub {
@@ -129,25 +142,28 @@ sub generator {
         }
 
         my $url;
-        until ( $url ) {
+        until ($url) {
             my $line = <$fh> // return;
             chomp $line;
             $line =~ s/^\s+|\s+$//g;
-            next if $line eq ''; # ignore empty lines
+            next if $line eq '';    # ignore empty lines
 
             my $request = eval { $self->request_hook($line) };
             $url = $self->construct_url($request);
-            warn "failed to construct URL: $line\n" if !$url and $self->warn;
+            if ( !$url ) {
+                warn "failed to construct URL: $line\n" if $self->warn;
+                $self->log->warn("failed to construct URL: $line");
+            }
         }
 
         $data = $self->request($url);
 
-        return (ref $data // '') eq 'ARRAY' ? shift @$data : $data;
-    }
+        return ( ref $data // '' ) eq 'ARRAY' ? shift @$data : $data;
+      }
 }
 
 sub request_hook {
-    my ($self, $line) = @_;
+    my ( $self, $line ) = @_;
     return $line =~ /^\s*{/ ? $self->json->decode($line) : $line;
 }
 
@@ -155,33 +171,37 @@ sub construct_url {
     my $self    = shift;
     my $url     = @_ > 1 ? _url_template_or_url(shift) : $self->url;
     my $request = shift;
-        
+
     # Template or query variables
-    if (ref $request and not blessed $request) {
+    if ( ref $request and not blessed $request) {
         return unless blessed $url;
-        if ($url->isa('URI::Template')) {
+        if ( $url->isa('URI::Template') ) {
             $url = $url->process($request);
-        } else {
+        }
+        else {
             $url = $url->clone;
             $url->query_form($request);
-        }  
-        return $url;       
-    } elsif (blessed $request and $request->isa('URI::URL')) {
+        }
+        return $url;
+    }
+    elsif ( blessed $request and $request->isa('URI::URL') ) {
         return $request;
-    } elsif ( $request =~ /^https?:\/\// ) { # plain URL
+    }
+    elsif ( $request =~ /^https?:\/\// ) {    # plain URL
         return URI->new($request);
-    } elsif ( $request  =~ /^\// ) { # URL path (and optional query)
+    }
+    elsif ( $request =~ /^\// ) {             # URL path (and optional query)
         $url = "$url";
-        $url =~ s{/$}{}; 
+        $url =~ s{/$}{};
         $request =~ s{\s+$}{};
-        return URI->new($url . $request);
+        return URI->new( $url . $request );
     }
 
     return;
-} 
+}
 
 sub request {
-    my ($self, $url) = @_;
+    my ( $self, $url ) = @_;
 
     $self->log->debug($url);
 
@@ -193,33 +213,36 @@ sub request {
 
     if ( $self->cache ) {
         $json = $self->cache->get($url);
-        if (defined $json) {
+        if ( defined $json ) {
             return ref $json ? $json : undef;
-        }   
+        }
     }
 
     if ( $self->wait and $self->time ) {
-        my $elapsed = ($self->time // time) - time;
+        my $elapsed = ( $self->time // time ) - time;
         sleep( $self->wait - $elapsed );
     }
     $self->time(time);
 
-    my $response = $self->client->get($url, $self->headers);
-    if ($response->is_success) {
+    my $response = $self->client->get( $url, $self->headers );
+    if ( $response->is_success ) {
         my $content = $response->decoded_content;
         my $data    = $self->json->decode($content);
-        $json       = $self->response_hook($data);
-     } else {
-        warn "request failed: $url\n" unless !$self->warn;
-        if ($response->status =~ /^4/) {
+        $json = $self->response_hook($data);
+    }
+    else {
+        warn "request failed: $url\n" if $self->warn;
+        $self->log->warn("request failed: $url");
+        if ( $response->status =~ /^4/ ) {
             $json = '';
-        } else {
+        }
+        else {
             return;
         }
     }
 
     if ( $self->cache ) {
-        $self->cache->set($url, $json);
+        $self->cache->set( $url, $json );
     }
 
     return ref $json ? $json : undef;
@@ -256,7 +279,7 @@ The following three examples are equivalent:
         url  => "http://example.org",
         file => \"/alice.json\n/bob.json"
     )->each(sub { my ($record) = @_; ... );
-    
+
     Catmandu::Importer::getJSON->new(
         url  => "http://example.org/{name}.json",
         file => \"{\"name\":\"alice\"}\n{\"name\":\"bob\"}"
@@ -289,7 +312,7 @@ A line that starts with "C</>" is appended to the configured B<url> parameter.
 
 A JSON object with variables to be used with an URL template or as HTTP query
 parameters. For instance the input line C<< {"name":"Karl Marx"} >> with URL
-C<http://api.lobid.org/person> or the input line 
+C<http://api.lobid.org/person> or the input line
 C<< {"entity":"person","name":"Karl Marx"} >> with URL template
 C<http://api.lobid.org/{entity}{?id}{?name}{?q}> are both expanded to
 L<http://api.lobid.org/person?name=Karl+Marx>.
@@ -306,7 +329,7 @@ item.
 
 =item url
 
-An L<URI> or an URI templates (L<URI::Template>) as defined by 
+An L<URI> or an URI templates (L<URI::Template>) as defined by
 L<RFC 6570|http://tools.ietf.org/html/rfc6570> to load JSON from. If no B<url>
 is configured, plain URLs must be provided as input or option C<from> must be
 used instead.
@@ -325,7 +348,7 @@ Instance of a L<Furl> HTTP client to perform requests with.
 
 =item dry
 
-Don't do any HTTP requests but return URLs that data would be queried from. 
+Don't do any HTTP requests but return URLs that data would be queried from.
 
 =item file / fh
 
@@ -351,6 +374,10 @@ enable global in-memory-caching, or a false value to disable caching (default).
 File caching uses file names based on MD5 of an URL so for instance
 C<http://example.org/> is cached as C<4389382917e51695b759543fdfd5f690.json>.
 
+=item warn
+
+Show error messages on the standard error.
+
 =back
 
 =head1 METHODS
@@ -365,8 +392,8 @@ instance to add timestamps or the measure how fast requests were responded.
 Returns an URL given a hash reference with variables, a plain URL or an URL
 path. The optional first argument can be used to override option C<url>.
 
-    $importer->construct_url( %query_vars ) 
-    $importer->construct_url( $importer->url, %query_vars ) # equivalent 
+    $importer->construct_url( %query_vars )
+    $importer->construct_url( $importer->url, %query_vars ) # equivalent
 
 =head2 request($url)
 
@@ -382,7 +409,7 @@ respectively. See L<Catmandu::Importer::Wikidata> for an example.
 
 Gets a whitespace-trimmed input line and is expected to return an unblessed
 hash reference, an URL, or undef. Errors are catched and treated equal to
-undef. 
+undef.
 
 =head2 response_hook
 
